'use client'
import { useState, MouseEvent, useEffect } from 'react'
import styles from './Board.module.css'
import { TSquare } from '@/types/TSquare'
import Square from '../Square/Square'
import { getRandomNumber } from '@/helpers/getRandomNumber'

type Props = {
    boardX: number
    boardY: number
    width: number
    height: number
}
const Board = ({boardX, boardY, width, height}: Props) => {
    const [board, setBoard] = useState<TSquare[][]>([])
    const [isFound, setIsFound] = useState<boolean>(false)
    const [counter, setCounter] = useState<number>(0)
    const [gameCount, setGameCount] = useState<number>(0)

    const initBoard = () => {
        const rnd = getRandomNumber (0, boardY * boardX - 1)
        const finalBoard: TSquare[][] = []
        for (let i = 0; i < boardY; i++) {
            const row: TSquare[] = []
            for (let j = 0; j < boardX; j++) {
                const treasure = boardX * i + j === rnd
                row.push(
                    {
                        key: i + ' ' + j + ' ' + gameCount,
                        isTreasure: treasure,
                        width,
                        height
                    }
                )
            }
            finalBoard.push(row)
        }
        setBoard(finalBoard)
    }

    useEffect(() => {
        initBoard()
        return () => {
            setBoard([])
            setIsFound(false)
            setCounter(0)
        }
    }, [])

    const squareClickHandler = (e: MouseEvent<HTMLDivElement>, isTreasure: boolean) => {
        if (!e.currentTarget || isFound) return
        console.log(isFound)
        console.log('TREAUSRE: ', isTreasure)
        setCounter(prevState => {
            return prevState + 1
        })
        if (isTreasure) {
            e.currentTarget.style.background = 'transparent'
            e.currentTarget.innerText = 'O'
            setIsFound(true)
            return
        }
        e.currentTarget.style.background = 'transparent'
    }
    const reset = () => {
        setBoard([])
        setCounter(0)
        setIsFound(false)
        initBoard()
        setGameCount(prevState => {
            return prevState + 1
        })
    }

    return (
        <div style={{
            gridTemplateColumns: `repeat(${boardX}, ${width}px)`,
            maxWidth: `${(width  + 2) * boardX + 40}px`,
        }} className={styles.board}>
            {board.map(row => {
                return row.map(sq => {
                    return <Square 
                                key={sq.key}
                                isTreasure={sq.isTreasure}
                                width={sq.width}
                                height={sq.height}
                                onClick={(e) => {squareClickHandler(e, sq.isTreasure)}}
                            />
                })
            })}
            <h2>{counter}</h2>
            <button onClick={reset}>RESET</button>
        </div>
    )
}

export default Board